#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/syscall.h>
#include <pthread.h>
#include <errno.h>

typedef struct {
	pthread_t thread;
} thread_param_t;

/* スレッドID を取得する */
pid_t gettid(void)
{
	/*
	 *  glibc のラッパー関数が存在しないので間接システムコールを用いる
	 */
	pid_t tid;

	tid = syscall(SYS_gettid);

	return tid;
}

/* スレッド */
void *thread_test(void *args)
{
	int ret;
	int tid;
	int policy;
	int prio[3] = {19, -1, 0};
	struct sched_param schedParam;
	thread_param_t *thread_param = args;
	char sched_msg[] = "cat /proc/%d/sched | grep -A 1 policy";
	char msg[80] = {0};
	int i;

	tid = (int)gettid();
	printf("thread_test: pid = %d tid= %d\n", getpid(), tid);
	sprintf(msg, sched_msg, tid);
	system(msg);

	/* スレッドのスケージューリングパラメータを取得する */
	ret = pthread_getschedparam(thread_param->thread, &policy, &schedParam);
	if (ret != 0) {
		fprintf(stderr, "pthread_getschedparam = %d\n", ret);
	}
	printf("policy = %d, sched_priority = %d\n", policy, schedParam.sched_priority);

	for (i=0; i<3; i++) {
		printf("change nice = %d\n", prio[i]);

		/* スレッドプライオリティの変更 */
		errno = 0;
		ret = nice(prio[i]);
		if (errno != 0) {
			perror("nice");
		}
		system(msg);

		/* キー入力待ち */
		getchar();
	}

	/* スレッドの終了 */
	pthread_exit(NULL);
}

/* メイン */
int main(void)
{
	int ret;
	pthread_attr_t attr;
	struct sched_param schedParam;
	thread_param_t thread_param;

	printf("my pid = %d\n", getpid());
#if 0
	/* スレッド属性オブジェクトの初期化と破棄を行う */
	ret = pthread_attr_init(&attr);
	if (ret != 0) {
		fprintf(stderr, "pthread_attr_init = %d\n", ret);
	}

	/* inherit-scheduler 属性の設定/取得を行う */
	ret = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED);
	if (ret != 0) {
		fprintf(stderr, "pthread_attr_setinheritsched = %d\n", ret);
	}

	/* スケジューリングポリシー属性の設定を行う */
	ret = pthread_attr_setschedpolicy(&attr, SCHED_RR);
	if (ret != 0) {
		fprintf(stderr, "pthread_attr_setschedpolicy = %d\n", ret);
	}

	/* スケジューリングパラメーター属性の設定を行う */
	schedParam.sched_priority = 10;
	ret = pthread_attr_setschedparam(&attr, &schedParam);
	if (ret != 0) {
		fprintf(stderr, "pthread_attr_setschedparam = %d\n", ret);
	}
#endif
	/* 新しいスレッドを作成する */
//	ret = pthread_create(&thread_param.thread, &attr, thread_test, &thread_param);
	ret = pthread_create(&thread_param.thread, NULL, thread_test, &thread_param);
	if (ret != 0) {
		fprintf(stderr, "pthread_create = %d\n", ret);
	}

	/* 終了したスレッドを join する */
	ret = pthread_join(thread_param.thread, NULL);
	if (ret != 0) {
		fprintf(stderr, "pthread_join = %d\n", ret);
	}

	return 0;
}
